<?php

namespace wpopt;

class Config
{
    static $plugin_name = 'wpopt';
    static $plugin_version;
    static $plugin_version_name;
    static $plugin_dir_name;
    static $plugin_core_dir;
    static $plugin_dir;
    static $plugin_url;
    static $plugin_static_url;
    static $plugin_static_dir;
    static $plugin_img_dir;
    static $plugin_css_url;
    static $plugin_js_url;
    static $plugin_lib_url;
    static $plugin_server_url;


    static function init()
    {
        self::$plugin_dir_name = 'wp-opt';
        self::$plugin_dir = WP_PLUGIN_DIR . '/' . self::$plugin_dir_name;
        self::$plugin_url = WP_PLUGIN_URL . '/' . self::$plugin_dir_name;
        self::$plugin_static_url = WP_PLUGIN_URL . '/' . self::$plugin_dir_name . '/static';
        self::$plugin_static_dir = self::$plugin_dir . '/static';
        self::$plugin_img_dir = self::$plugin_static_dir . '/img';
        self::$plugin_css_url = self::$plugin_static_url . '/css';
        self::$plugin_js_url = self::$plugin_static_url . '/js';
        self::$plugin_lib_url = self::$plugin_static_url . '/lib';
        self::$plugin_version = 5;
        self::$plugin_version_name = '1.4.0';
        self::$plugin_core_dir = self::$plugin_dir . '/core';
        self::$plugin_server_url = 'https://interface.lovestu.com/api?n=wpopt';
        self::loadFile();
    }

    static function loadFile()
    {
        require_once self::$plugin_core_dir . '/class-wordpress.php';
        require_once self::$plugin_core_dir . '/class-load-file.php';
        require_once self::$plugin_core_dir . '/class-tools.php';
        require_once self::$plugin_core_dir . '/class-plugin.php';
        require_once self::$plugin_core_dir . '/class-ajax.php';

        Plugin::init();
        LoadFile::init();
        Ajax::init();
        WordPress::init();
    }

    static function getAdminSetData($hook)
    {

        $data['ajax_url'] = admin_url('admin-ajax.php');
        $data['ajax_name'] = Config::$plugin_name;
        $data['version_name'] = Config::$plugin_version_name;
        $data['set'] = self::getSet();
        if (time() - $data['set']['update']['last_check_time'] > 60 * 60) {
            $data['can_check_update'] = 1;
        } else {
            $data['can_check_update'] = 0;
        }
        $data['need_update'] = $data['set']['update']['need_update'];
        $data['hook'] = $hook;

        return $data;
    }


    static function getSet()
    {
        $set = get_option(self::$plugin_name . '_set');
        if ($set === false) {
            $set = self::_getSetDefault();
        } else {
            $set = json_decode($set, true);
            $set = self::updateSet($set);
        }
        return $set;
    }

    static function saveSet($data)
    {
        return update_option(self::$plugin_name . '_set', json_encode($data));
    }

    static function updateSet($set)
    {
        $default_set = self::_getSetDefault();

        foreach ($default_set as $key => $value) {
            foreach ($value as $key2 => $value2) {
                if (isset($set[$key][$key2])) {
                    $default_set[$key][$key2] = $set[$key][$key2];
                }
            }
        }
        return $default_set;
    }

    static function _getSetDefault()
    {
        $data['echo']['open'] = 1;
        $data['echo']['remove_version'] = 0;
        $data['echo']['remove_file_version'] = 0;
        $data['echo']['remove_dns_prefetch'] = 0;
        $data['echo']['remove_json_url'] = 0;
        $data['echo']['remove_post_meta'] = 0;
        $data['echo']['remove_post_feed'] = 0;
        $data['echo']['remove_wp_block_library_css'] = 0;
        $data['echo']['remove_dashicons'] = 0;
        $data['echo']['remove_wlwmanifest'] = 0;
        $data['echo']['remove_rsd'] = 0;
        $data['echo']['remove_short_link'] = 0;

        $data['fun_ban']['open'] = 1;
        $data['fun_ban']['translations_api'] = 0;

        $data['fun_ban']['wp_check_php_version'] = 0;
        $data['fun_ban']['wp_check_browser_version'] = 0;
        $data['fun_ban']['current_screen'] = 0;


        $data['fun_switch']['open'] = 1;
        $data['fun_switch']['rest_api'] = 0;
        $data['fun_switch']['pingback'] = 0;
        $data['fun_switch']['xml_rpc'] = 0;

        $data['fun_switch']['emoji'] = 0;
        $data['fun_switch']['admin_bar'] = 0;
        $data['fun_switch']['login_translate'] = 0;


        $data['fun_switch']['revision'] = 0;
        $data['fun_switch']['image_height_limit'] = 0;
        $data['fun_switch']['image_creat_size'] = 0;
        $data['fun_switch']['remove_image_attributes'] = 0;


        $data['fun_switch']['transcoding'] = 0;
        $data['fun_switch']['auto_embeds'] = 0;
        $data['fun_switch']['post_embeds'] = 0;
        $data['fun_switch']['gutenberg'] = 0;

        $data['fun_switch']['widget_gutenberg'] = 0;


        $data['fun_switch']['core_update'] = 0;
        $data['fun_switch']['theme_update'] = 0;
        $data['fun_switch']['plugin_update'] = 0;

        $data['fun_switch']['mail_update_user_info_note'] = 0;
        $data['fun_switch']['mail_register_note'] = 0;
        $data['fun_switch']['email_check'] = 0;

        $data['optimization']['open'] = 1;
        $data['optimization']['gravatar'] = 'no';

        $data['update']['need_update'] = 0;
        $data['update']['last_check_time'] = time();

        $data['fun_enhance']['open'] = 1;
        $data['fun_enhance']['webp_support'] = 0;
        $data['fun_enhance']['svg_support'] = 0;
        $data['fun_enhance']['page_add_html'] = 0;
        $data['fun_enhance']['remove_link_category_text'] = 0;

        $data['fun_enhance']['paste_upload_img'] = 0;
        $data['fun_enhance']['paste_upload_img_to_webp'] = 0;

        return $data;
    }
}