<?php

namespace wpopt;
class WordPress
{
    static $set;

    static function init()
    {
        self::$set = Config::getSet();
        if (self::$set['echo']['open'] === 1) {
            self::setEcho();
        }
        if (self::$set['fun_ban']['open'] === 1) {
            self::setFunBan();
        }
        if (self::$set['fun_switch']['open'] === 1) {
            self::setFunSwitch();
        }
        if (self::$set['optimization']['open'] === 1) {
            self::setOptimization();
        }
        if (self::$set['fun_enhance']['open'] === 1) {
            self::setFunEnhance();
        }
    }

    static function setOptimization()
    {
        $set = self::$set;
        if ($set['optimization']['gravatar'] != 'no') {
            add_filter('get_avatar_url', array(static::class, 'replaceAvatar'), 10, 3);
        }
    }

    static function setEcho()
    {
        $set = self::$set;
        if ($set['echo']['remove_version'] == 1) {
            // 头部
            remove_action('wp_head', 'wp_generator');
            //rss
            add_filter('the_generator', '__return_empty_string');
        }
        if ($set['echo']['remove_file_version'] == 1) {
            add_filter('style_loader_src', array(static::class, 'removeFileVersion'), 9999);
            add_filter('script_loader_src', array(static::class, 'removeFileVersion'), 9999);
        }
        if ($set['echo']['remove_dns_prefetch'] == 1) {
            remove_action('wp_head', 'wp_resource_hints', 2);
        }
        if ($set['echo']['remove_json_url'] === 1) {
            //去除json连接
            remove_action('wp_head', 'rest_output_link_wp_head');
            remove_action('template_redirect', 'rest_output_link_header');
        }
        if ($set['echo']['remove_post_meta'] === 1) {
            //移除前后文、第一篇文章、主页meta信息
            remove_action('wp_head', 'adjacent_posts_rel_link_wp_head');
        }
        if ($set['echo']['remove_post_feed'] === 1) {
            //移除feed
            remove_action('wp_head', 'feed_links', 2);//文章和评论feed
            remove_action('wp_head', 'feed_links_extra', 3); //分类等feed
        }
        if ($set['echo']['remove_wp_block_library_css'] === 1) {
            //WordPress 5.0+移除 古藤堡编辑器
            add_action('wp_enqueue_scripts', array(static::class, 'removeWpBlockLibraryCss'), 9999);
        }
        if ($set['echo']['remove_dashicons'] === 1) {
            add_action('wp_print_styles', array(static::class, 'removeDashicons'), 9999);
        }
        if ($set['echo']['remove_wlwmanifest'] === 1) {
            remove_action('wp_head', 'wlwmanifest_link');
        }
        if ($set['echo']['remove_rsd'] === 1) {
            remove_action('wp_head', 'rsd_link');
        }
        if ($set['echo']['remove_short_link'] === 1) {
            remove_action('wp_head', 'wp_shortlink_wp_head');
        }


    }

    static function setFunBan()
    {
        $set = self::$set;
        if ($set['fun_ban']['translations_api'] === 1) {
            add_filter('translations_api', '__return_true');
        }
        if ($set['fun_ban']['wp_check_php_version'] == 1) {
            add_filter('pre_site_transient_{$transient}', '__return_true');
        }
        if ($set['fun_ban']['wp_check_browser_version'] == 1) {
            add_filter('wp_check_browser_version', '__return_false');
        }
        if ($set['fun_ban']['current_screen'] == 1) {
            add_filter('current_screen', '__return_false');
        }
    }

    static function replaceAvatar($avatarUrl, $id_or_email, $arrs)
    {
        $set = self::$set;
        if ($set['optimization']['gravatar'] == 'v2ex') {
            $cdnurl = 'cdn.v2ex.com/gravatar';
            $avatarUrl = self::_replaceGravatar($cdnurl, $avatarUrl);
        } elseif ($set['optimization']['gravatar'] == 'geek') {
            $cdnurl = 'sdn.geekzu.org/avatar';
            $avatarUrl = self::_replaceGravatar($cdnurl, $avatarUrl);
        } elseif ($set['optimization']['gravatar'] == 'cn') {
            $cdnurl = 'cn.gravatar.com/avatar';
            $avatarUrl = self::_replaceGravatar($cdnurl, $avatarUrl);
        } elseif ($set['optimization']['gravatar'] == 'qiniu') {
            $cdnurl = 'dn-qiniu-avatar.qbox.me/avatar';
            $avatarUrl = self::_replaceGravatar($cdnurl, $avatarUrl);
        } elseif ($set['optimization']['gravatar'] == 'loli') {
            $cdnurl = 'gravatar.loli.net/avatar';
            $avatarUrl = self::_replaceGravatar($cdnurl, $avatarUrl);
        } elseif ($set['optimization']['gravatar'] == 'chinayes') {
            $cdnurl = 'cravatar.cn/avatar';
            $avatarUrl = self::_replaceGravatar($cdnurl, $avatarUrl);
        }
        return $avatarUrl;
    }

    static function _replaceGravatar($url, $avatarUrl)
    {
        $avatarUrl = str_replace(array("secure.gravatar.com/avatar", "www.gravatar.com/avatar", "0.gravatar.com/avatar", "1.gravatar.com/avatar", "2.gravatar.com/avatar", "cn.gravatar.com/avatar"), $url, $avatarUrl);
        return $avatarUrl;
    }

    static function setFunSwitch()
    {
        $set = self::$set;
        if ($set['fun_switch']['rest_api'] === 1) {
            //屏蔽 REST API
            add_filter('rest_enabled', '__return_false');
            add_filter('rest_jsonp_enabled', '__return_false');
        }
        if ($set['fun_switch']['pingback'] === 1) {
            add_filter('xmlrpc_methods', function ($methods) {
                $methods['pingback.ping'] = '__return_false';
                $methods['pingback.extensions.getPingbacks'] = '__return_false';
                return $methods;
            });
            //禁用 pingbacks, enclosures, trackbacks
            remove_action('do_pings', 'do_all_pings');
            //去掉 _encloseme 和 do_ping 操作。
            remove_action('publish_post', '_publish_post_hook');
        }
        if ($set['fun_switch']['xml_rpc'] === 1) {
            add_filter('xmlrpc_enabled', '__return_false');
            add_filter('xmlrpc_methods', '__return_empty_array');
        }

        if ($set['fun_switch']['emoji'] === 1) {
            //禁止emoji
            remove_action('admin_print_scripts', 'print_emoji_detection_script');
            remove_action('admin_print_styles', 'print_emoji_styles');
            remove_action('wp_head', 'print_emoji_detection_script', 7);
            remove_action('wp_print_styles', 'print_emoji_styles');
            remove_action('embed_head', 'print_emoji_detection_script');
            remove_filter('the_content_feed', 'wp_staticize_emoji');
            remove_filter('comment_text_rss', 'wp_staticize_emoji');
            remove_filter('wp_mail', 'wp_staticize_emoji_for_email');
        }
        if ($set['fun_switch']['admin_bar'] === 1) {
            add_filter('show_admin_bar', '__return_false');
        }
        if ($set['fun_switch']['login_translate'] === 1) {
            add_filter('login_display_language_dropdown', '__return_false');
        }
        if ($set['fun_switch']['revision'] === 1) {
            define('WP_POST_REVISIONS', false);
        }
        if ($set['fun_switch']['image_height_limit'] === 1) {
            add_filter('big_image_size_threshold', '__return_false');
        }
        if ($set['fun_switch']['image_creat_size'] === 1) {
            add_filter('intermediate_image_sizes_advanced', array(static::class, 'BanCustomizeImageSizes'));
            add_filter('big_image_size_threshold', '__return_false');
        }
        if ($set['fun_switch']['remove_image_attributes'] === 1) {
            add_filter('post_thumbnail_html', array(static::class, 'removeImagesAttribute'), 10);
            add_filter('image_send_to_editor', array(static::class, 'removeImagesAttribute'), 10);
        }

        if ($set['fun_switch']['transcoding'] === 1) {
            add_filter('run_wptexturize', '__return_false');
        }
        if ($set['fun_switch']['auto_embeds'] === 1) {
            remove_filter('the_content', array($GLOBALS['wp_embed'], 'autoembed'), 8);
        }

        if ($set['fun_switch']['post_embeds'] === 1) {
            remove_action('wp_head', 'wp_oembed_add_discovery_links');
            remove_action('wp_head', 'wp_oembed_add_host_js');
        }

        if ($set['fun_switch']['gutenberg'] === 1) {
            add_filter('use_block_editor_for_post', '__return_false');
        }
        if ($set['fun_switch']['widget_gutenberg'] === 1) {
            add_filter('gutenberg_use_widgets_block_editor', '__return_false');
            add_filter('use_widgets_block_editor', '__return_false');
        }

        if ($set['fun_switch']['core_update'] === 1) {
            // 彻底关闭自动更新
            add_filter('automatic_updater_disabled', '__return_true');
            // 关闭更新检查定时作业
            remove_action('init', 'wp_schedule_update_checks');
            //  移除已有的版本检查定时作业
            wp_clear_scheduled_hook('wp_version_check');
            // 移除已有的自动更新定时作业
            wp_clear_scheduled_hook('wp_maybe_auto_update');
            // 移除后台内核更新检查
            remove_action('admin_init', '_maybe_update_core');
            add_filter('pre_site_transient_update_core', function ($a) {
                return null;
            });
        }
        if ($set['fun_switch']['theme_update'] === 1) {
            wp_clear_scheduled_hook('wp_update_themes');
            add_filter('auto_update_theme', '__return_false');
            remove_action('load-themes.php', 'wp_update_themes');
            remove_action('load-update.php', 'wp_update_themes');
            remove_action('load-update-core.php', 'wp_update_themes');
            remove_action('admin_init', '_maybe_update_themes');
            add_filter('pre_site_transient_update_plugins', function ($a) {
                return null;
            });
        }

        if ($set['fun_switch']['plugin_update'] === 1) {
            wp_clear_scheduled_hook('wp_update_plugins');
            add_filter('auto_update_plugin', '__return_false');
            add_filter('pre_site_transient_update_themes', function ($a) {
                return null;
            });
            remove_action('load-plugins.php', 'wp_update_plugins');
            remove_action('load-update.php', 'wp_update_plugins');
            remove_action('load-update-core.php', 'wp_update_plugins');
            remove_action('admin_init', '_maybe_update_plugins');
        }

        if ($set['fun_switch']['mail_update_user_info_note'] === 1) {
            add_filter('send_password_change_email', '__return_false');
            add_filter('email_change_email', '__return_false');
            if (!function_exists('wp_password_change_notification')) {
                function wp_password_change_notification($user)
                {
                    return null;
                }
            }
        }
        if ($set['fun_switch']['mail_register_note'] === 1) {
            add_filter('wp_new_user_notification_email_admin', '__return_false');
            add_filter('wp_new_user_notification_email', '__return_false');
        }
        if ($set['fun_switch']['email_check'] === 1) {
            add_filter('admin_email_check_interval', '__return_false');
        }


    }

    static function setFunEnhance()
    {
        $set = self::$set;
        if ($set['fun_enhance']['webp_support'] == 1) {
            add_filter('upload_mimes', array(static::class, 'uploadSupport'));
            add_filter('file_is_displayable_image', array(static::class, 'fileDisplaySupport'), 10, 2);
        }
        if ($set['fun_enhance']['svg_support'] == 1) {
            add_filter('upload_mimes', array(static::class, 'uploadSupport'));
            add_filter('file_is_displayable_image', array(static::class, 'fileDisplaySupport'), 10, 2);
        }
        if ($set['fun_enhance']['page_add_html'] == 1) {
            add_action('init', array(static::class, 'rewritePage'), -1);
            add_filter('user_trailingslashit', array(static::class, 'noPageSlash'), 100, 2);
        }
        if ($set['fun_enhance']['paste_upload_img'] == 1) {
            add_filter('tiny_mce_before_init', array(static::class, 'tinyMCEEnhance'));

        }
    }



    static function tinyMCEEnhance($settings)
    {
        $settings['paste_data_images'] = false;
        //$settings['paste_postprocess'] = 'function(editor, args){wpopt_paste_postprocess(editor, args)}';

        return $settings;
    }

    static function zm_category_link($catlink, $category_id)
    {
        global $wp_rewrite;
        $catlink = $wp_rewrite->get_category_permastruct();

        if (empty($catlink)) {
            $catlink = home_url('?cat=' . $category_id);
        } else {
            $category = get_category($category_id);
            $category_nicename = $category->slug;

            $catlink = str_replace('%category%', $category_nicename, $catlink);
            $catlink = home_url(user_trailingslashit($catlink, 'category'));
        }
        return $catlink;
    }

    static public function rewritePage()
    {
        global $wp_rewrite;

        if (!strpos($wp_rewrite->get_page_permastruct(), '.html')) {
            $wp_rewrite->page_structure = $wp_rewrite->page_structure . '.html';
        }
    }

    static public function noPageSlash($string, $type)
    {
        global $wp_rewrite;

        if ($wp_rewrite->using_permalinks() && $wp_rewrite->use_trailing_slashes == true && $type == 'page') {
            return untrailingslashit($string);
        }
        return $string;
    }

    static function fileDisplaySupport($result, $path)
    {
        $set = self::$set;
        $info = @getimagesize($path);
        if ($set['fun_enhance']['webp_support'] == 1) {
            if ($info['mime'] == 'image/webp') {
                $result = true;
            }
        }
        if ($set['fun_enhance']['svg_support'] == 1) {
            if ($info['mime'] == 'image/svg+xml') {
                $result = true;
            }
        }
        return $result;
    }

    static function uploadSupport($mimes = array())
    {
        $set = self::$set;

        if ($set['fun_enhance']['svg_support'] == 1) {
            $mimes['svg'] = 'image/svg+xml';
        }
        if ($set['fun_enhance']['webp_support'] == 1) {
            if (floatval(get_bloginfo('version')) < 5.8) {
                $mimes['webp'] = 'image/webp';
            }
        }
        return $mimes;
    }

    static function BanCustomizeImageSizes($sizes)
    {
        return [];
    }

    static function removeImagesAttribute($html)
    {
        $html = preg_replace('/width="(\d*)"\s+height="(\d*)"\s+class=\"[^\"]*\"/', "", $html);
        $html = preg_replace('/  /', "", $html);
        return $html;
    }

    static function removeWpBlockLibraryCss()
    {
        wp_dequeue_style('wp-block-library');
        wp_dequeue_style('wp-block-library-theme');
        wp_dequeue_style('wc-blocks-style');
    }

    static function removeDashicons()
    {
        if (!is_admin_bar_showing() && !is_customize_preview()) {
            wp_dequeue_style('dashicons');
            wp_deregister_style('dashicons');
        }
    }

    static function removeFileVersion($src)
    {
        if (strpos($src, 'ver=')) {
            $src = remove_query_arg('ver', $src);
        }
        return $src;
    }

}